#!/usr/bin/env python

from setuptools import setup, find_packages
import versioneer


requires = open('requirements.txt').read().strip().split('\n')

setup(
    name='xrviz',
    version=versioneer.get_version(),
    cmdclass=versioneer.get_cmdclass(),
    description='Interactive visualisations for xarrays',
    url='',
    maintainer='Martin Durant',
    maintainer_email='mdurant@anaconda.com',
    license='BSD',
    py_modules=['xrviz'],
    packages=find_packages(),
    package_data={'': ['*.csv', '*.yml', '*.html']},
    include_package_data=True,
    install_requires=requires,
    long_description=open('README.md').read(),
    zip_safe=False,
    entry_points={
        'console_scripts': [
            'xrviz = xrviz.cli:main'
        ]},
)
